#
/*
 *		M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *			      Douglas P. Kingston III
 *
 *				  Q L O C K S . C
 *
 *	-----------------------------------------------------------------------
 *
 *				Revision History
 *				================
 */

#include <stdio.h>
#include <errno.h>
#include "queue.h"		/* Includes <sys/types.h> */
#include <sys/file.h>

extern	lock_t *All_locks[];

/*
 *		L O C K  ( )
 */
lock (lockname, waittime)
lock_t *lockname;
int	waittime;
{
	if (lockname->lfd != -1) {
		/* The lock is already locked */
		return (0);
	}
	while ((lockname->lfd = open (lockname->lockfile, 
	       O_RDONLY | O_NDELAY )) < 0) {
		if (errno == ENOENT) {
			if (close (creat (lockname->lockfile, 0444)) < 0)
				return (-1);	/* Not there and can't make */
			chmod (lockname->lockfile, 0444);  /* overide umask */
		} else {
			err ("open error on %s (%s)",
				  lockname->lockfile, sys_errlist[errno]);
			return (-1);
		}
	}
	while (flock (lockname->lfd, LOCK_EX|LOCK_NB) < 0) {
		if (errno == EWOULDBLOCK) {
			if( waittime-- > 0 )
				sleep (1);	/* Lock is in use, wait */
			else {
				close (lockname->lfd);
				return( -1 );
			}
		} else {
			err ("Lock error on %s (%s)",
				  lockname->lockfile, sys_errlist[errno]);
			close (lockname->lfd);
			return (-1);
		}

	}
	return (0);
}

/*
 *		U N L O C K  ( )
 */
unlock (lockname)
lock_t *lockname;
{
	if (lockname->lfd != -1)
		flock (lockname->lfd, LOCK_UN);
}

/*
 *		F R E E L O C K S  ( )
 *  Release all locks held by this process.
 */
freelocks ()
{
	lock_t **lckp;

	lckp = All_locks;
	while( (*lckp) != 0 )
		unlock( *(lckp++) );
}

/*
 *		C L E A R L O C K  ( )
 *  This is a dangerous function.  It clears a lock by removing
 *  the current one and creating a new one.  If the lock is not
 *  in use, nothing is done.
 */
clearlock (lockname)
lock_t *lockname;
{
	if ((lockname->lfd = open (lockname->lockfile,
	     O_RDONLY | O_NDELAY)) < 0) {
		if (errno == ENOENT) {
			if (close (creat (lockname->lockfile, 0444)) < 0)
				return (-1);
			return (0);
		} else
			return (-1);
	}
	if (flock (lockname->lfd, LOCK_EX|LOCK_NB) != 0) {
		if (errno == EWOULDBLOCK) {
			if (unlink (lockname->lockfile) < 0)
				return (-1);
			if (close (creat (lockname->lockfile, 0444)) < 0)
				return (-1);
			return (0);
		} else 
			return (-1);
	}
	flock (lockname->lfd, LOCK_UN);
	close (lockname->lfd);
	return (0);
}
