#
/*
 *		M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *			      Douglas P. Kingston III
 *
 *				  Q L O C K S . C
 *
 *	-----------------------------------------------------------------------
 *
 *				Revision History
 *				================
 *
 *	02 Dec 82  DPK	Made modification to the link based locking inspired
 *			by the flock(III) package from Geotronics.  Warning:
 *			this modification requires that the lockfile names
 *			be no more than QDIRSIZ - 2 characters in size.
 *
 *	21 Dec 82  DPK	Added check for existence of lockfile, if missing,
 *			it will try to create it.
 */

#include <stdio.h>
#include <errno.h>
#include "queue.h"

extern	lock_t *All_locks[];

char	*LockName();
extern char	*rindex();
extern char	*xmalloc();

/*
 *		L O C K  ( )
 */
lock (lockname, waittime)
lock_t *lockname;
int	waittime;
{
	int	fd;
	char	*locklink;

	if (lockname->lfd != -1) {
		/* The lock is already locked */
		return (0);
	}

	if(( locklink = LockName( lockname->lockfile )) == NULL )
		return( -1 );		/* Can't make lockfile name */

	while ( link( lockname->lockfile, locklink ) < 0 ) {
		if( errno == EEXIST ) {
			if( waittime-- > 0 )
				sleep (1);	/* Lock is in use, wait */
			else {
				free( locklink );
				return( -1 );
			}
		} else if( errno == ENOENT ) {
			if(( fd = creat( lockname->lockfile, 0444 )) < 0 )
				return( -1 );
			close( fd );
		}
	}
	lockname->lfd = 0;
	free( locklink );
	return (0);
}

/*
 *		U N L O C K  ( )
 */
unlock (lockname)
lock_t *lockname;
{
	if (lockname->lfd != -1)
		unlink( LockName( lockname->lockfile ));
	lockname->lfd = -1;
}

/*
 *		F R E E L O C K S  ( )
 *  Release all locks held by this process.
 */
freelocks ()
{
	lock_t **lckp;

	lckp = All_locks;
	while( (*lckp) != 0 )
		unlock( *(lckp++) );
}

/*
 *		C L E A R L O C K  ( )
 *  This is a dangerous function.  It clears a lock by removing
 *  the current one.
 */
clearlock (lockname)
lock_t *lockname;
{
	if( unlink ( LockName( lockname->lockfile )) < 0 ) {
		err( "clearlock: cannot clear lock %s (%s)",
			lockname->lockfile, sys_errlist[errno] );
		return( -1 );
	}
	lockname->lfd = -1;
	return (0);
}

static char *
LockName( file )			/* constructs lock-file name */
char	*file;				/* basename */
{
	static char	ext[] = " L";	/* lock-file extension */
	register int	len;		/* length of isolated name */
	register char	*start; 	/* -> after path prefix */
	register char	*lock;		/* constructed lock-file name */

	if ( (start = rindex( file, '/' )) != NULL )
		++start;		/* skip over '/' */
	else
		start = file;

	if ( (len = strlen( start )) + sizeof ext - 1 > QDIRSIZ )
		return NULL;		/* name would be too long */

	lock = xmalloc( (unsigned)(start - file + len + sizeof ext) );
	if ( lock != NULL )
		strcat( strcpy( lock, file ), ext );

	return lock;
}
